/*
 * (C) Copyright 2011 Nuxeo SAS (http://nuxeo.com/) and contributors.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser General Public License
 * (LGPL) version 2.1 which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/lgpl.html
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * Contributors:
 *    Nuxeo - initial API and implementation
 *    Olivier Lacroix - External contribution
 */


function debugLog(msg) {
    if (NXDEBUG) {
        dump(msg + "\n");
        Components.classes[CONSOLE_CONTRACTID].getService(nsIConsoleService)
                .logStringMessage(msg + "\n");
    }
}

getMappings = function() {
    var listbox = document.getElementById("mappingsList");
    // listbox.clearSelection();
    while (listbox.getRowCount() > 0) {
        listbox.removeItemAt(0);
    }
    var mappingStr = "" + document.getElementById("p_nxeditMimeMapping").value;
    var map = getMapFromStr(mappingStr);
    var mtList = map.getKeys();
    for ( var i = 0; i < mtList.length; i++) {
        var mt = mtList[i];
        var mappingEdit = map.getCommandValue(mt);
        var mappingEditArgs = map.getArgValue(mt);

        listitem = listbox.appendItem(mt, mt);
        var cell = document.createElement("listcell");
        cell.setAttribute("label", mappingEdit);
        listitem.appendChild(cell);
        cell2 = document.createElement("listcell");
        cell2.setAttribute("label", mappingEditArgs);
        listitem.appendChild(cell2);
    }
    updateAcceptHeader();
    listbox.setRowCount(mtList.length);
}

removeMappings = function() {
    var listbox = document.getElementById("mappingsList");
    var mappingStr = "" + document.getElementById("p_nxeditMimeMapping").value;
    var map = getMapFromStr(mappingStr);
    for (i = 0; item = listbox.getSelectedItem(i); i++) {
        var key = item.getAttribute("label");
        map.removeKey(key);
    }
    document.getElementById("p_nxeditMimeMapping").value = getStrFromMap(map);
    getMappings();
}

addMapping = function(newMimeType, newEditor, newEditorArgs) {
    var mappingStr = "" + document.getElementById("p_nxeditMimeMapping").value;
    if (mappingStr.lastIndexOf(";") != mappingStr.length - 1)
        mappingStr = mappingStr + ";";
    var map = getMapFromStr(mappingStr);
    map.putTuple(newMimeType, newEditor, newEditorArgs);
    mappingStr = getStrFromMap(map);
    document.getElementById("p_nxeditMimeMapping").value = mappingStr;
}

editMapping = function() {
    var listbox = document.getElementById("mappingsList");
    var item = listbox.getSelectedItem(0);
    var mt = item.getAttribute("label");
    var cmd = item.childNodes[0].getAttribute("label");
    var args = item.childNodes[1].getAttribute("label");
    document.getElementById("newMimeType").value = mt;
    document.getElementById("newEditor").value = cmd;
    document.getElementById("newEditorArgs").value = args;
}

saveMappings = function() {
    addMapping(document.getElementById("newMimeType").value, document
            .getElementById("newEditor").value, document
            .getElementById("newEditorArgs").value);
    document.getElementById("newMimeType").value = "";
    document.getElementById("newEditor").value = "";
    document.getElementById("newEditorArgs").value = "";
    getMappings();
}

updateAcceptHeader = function() {
    var mappingStr = "" + document.getElementById("p_nxeditMimeMapping").value;
    debugLog("[NUXEO]: " + mappingStr);
    var map = getMapFromStr(mappingStr);
    setUpInitButtons(map);
}

isWindowsHost = function() {
    var xulRuntime = Components.classes["@mozilla.org/xre/app-info;1"]
            .getService(Components.interfaces.nsIXULRuntime);
    if (xulRuntime.OS == "WINNT")
        return true;
    return false;
}

addDefaultMSOMapping = function() {
    if (!isWindowsHost()) {
        alert("This is only supported on Win32 hosts");
        return;
    }
    var cmdString = "C:\\Program Files\\Nuxeo\\Nuxeo Live Edit MsOffice IE\\nuxeo-liveedit-msoffice-launcher.exe";
    var wrk = Components.classes["@mozilla.org/windows-registry-key;1"].createInstance(Components.interfaces.nsIWindowsRegKey);
    wrk.open(wrk.ROOT_KEY_LOCAL_MACHINE,"SOFTWARE",wrk.ACCESS_READ);
    if (wrk.hasChild("Nuxeo")) {
        var nuxeokey = wrk.openChild("Nuxeo",wrk.ACCESS_READ);
        if (nuxeokey.hasChild("NuxeoLiveEdit")) {
            var LiveEditkey = nuxeokey.openChild("NuxeoLiveEdit",wrk.ACCESS_READ);
            if (LiveEditkey.hasValue("InstalledPath")) {
                cmdString = LiveEditkey.readStringValue("InstalledPath")+"nuxeo-liveedit-msoffice-launcher.exe";
            }
            LiveEditkey.close();
        }
        nuxeokey.close();
    }
    wrk.close();
    var argString = "%s";
    addMapping("application/msword", cmdString, argString);
    addMapping("application/vnd.ms-excel", cmdString, argString);
    addMapping("application/vnd.ms-powerpoint", cmdString, argString);
    addMapping("application/vnd.openxmlformats-officedocument.wordprocessingml.document", cmdString, argString);
    addMapping("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", cmdString, argString);
    addMapping("application/vnd.openxmlformats-officedocument.presentationml.presentation", cmdString, argString);
    getMappings();
}

addDefaultOOMapping = function() {
    var cmdString = "C:\\Program Files\\OpenOffice.org 3\\program\\soffice.exe";
    if (!isWindowsHost()) {
        cmdString = "/usr/bin/soffice";
    }
    var argString = "macro:///LiveEditOOo.launcher.load(%s)";
    addMapping("application/vnd.oasis.opendocument.text", cmdString, argString);
    addMapping("application/vnd.oasis.opendocument.spreadsheet", cmdString,
            argString);
    addMapping("application/vnd.oasis.opendocument.presentation", cmdString,
            argString);
    addMapping("application/msword", cmdString, argString);
    addMapping("application/vnd.ms-excel", cmdString, argString);
    addMapping("application/vnd.ms-powerpoint", cmdString, argString);
    addMapping("application/vnd.openxmlformats-officedocument.wordprocessingml.document", cmdString, argString);
    addMapping("application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", cmdString, argString);
    addMapping("application/vnd.openxmlformats-officedocument.presentationml.presentation", cmdString, argString);
    getMappings();
}
